--[[
	ZOMG!
	Copyright (C) 2011  Christopher 'Kasoki' Kaster

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]--

---------------------------
--    global variables   --
---------------------------

local language = Inspect.System.Language()

-- set english as default language
if not lang[language] then
	lang[language] = lang["English"]
end

---------------------------
--     event handler     --
---------------------------

function update(params)
	local player = Inspect.Unit.Detail("player")
	local dealer = Inspect.Unit.Detail(params.caster)
	
	if not dealer then
		return
	end
	
	if player.name == dealer.name then
		if params.crit then
			if not params.damage then
				value = params.heal
			else
				value = params.damage
			end
			
			if value then
				if not zomg_highscore.val then			
					zomg_highscore.id = params.ability
					zomg_highscore.val = value
				else
					highscore = zomg_highscore.val
				
					if value > highscore then
						print(string.format(lang[language].highscore_new, highscore, value, params.abilityName))
						
						zomg_highscore.id = params.ability
						zomg_highscore.val = value
					end
				end
				
				zomg_count = zomg_count + 1
				zomg_totaldmg = zomg_totaldmg + value
				
				insert_skill(params.ability, value)
				
				if not zomg_config.frame_mode then
					print(string.format(lang[language].message, params.abilityName, value))
				end
				
			end
		end
	end
end

function var_loaded()
	if not zomg_highscore then
		zomg_highscore = {}
	end
	
	if not zomg_skills then
		zomg_skills = {}
	end
	
	if not zomg_count then
		zomg_count = 0
	end
	
	if not zomg_totaldmg then
		zomg_totaldmg = 0
	end
	
	if not zomg_config then
		zomg_config = {}
		zomg_config.frame_mode = false
	end
end

function slash_register(param)
	local cmd = param:match("^(%a+)")
	local params = string.gsub(param, "^(%a+)(%s*)", "")
	
	-- print highscore statistic
	if (cmd == "highscore") or (cmd == "hs") then
		if not zomg_highscore.val then
			print(lang[language].nodata)
		else
			hs = zomg_highscore.val
			ability = Inspect.Ability.Detail(zomg_highscore.id)
			print(string.format(lang[language].highscore_text, hs, ability.name))
		end
	-- print average statistics
	elseif (cmd == "average") or (cmd == "avg") then
		if not (zomg_totaldmg > 0) then
			print(lang[language].nodata)
		else
			avg = math.floor(zomg_totaldmg / zomg_count * 100)/100
		
			print(string.format(lang[language].average_text, avg))
		end
	-- print common statistics
	elseif (cmd == "statistics") or (cmd == "stat") then
		print_skill_stats()
	-- print informations about ZOMG!
	elseif (cmd == "info") then
		print(lang[language].info)
	-- show zomg options
	elseif (cmd == "framemode") or (cmd == "frame") then
		if zomg_config.frame_mode then
			zomg_config.frame_mode = false
			print(lang[language].frame_mode.disabled)
		else
			zomg_config.frame_mode = true
			print(lang[language].frame_mode.enabled)
		end
	-- reset crit stats
	elseif (cmd == "reset") then
		reset()
	else
		print(lang[language].help_message)
	end
end

---------------------------
--         utils         --
---------------------------

function insert_skill(skill_id, dmg)
	found = false
	
	
	
	for index, skill in pairs(zomg_skills) do
		if skill.id == skill_id then			
			skill.totaldmg = skill.totaldmg + dmg
			skill.count = skill.count + 1
			
			if dmg > skill.highscore then
				skill.highscore = dmg
			end
			
			found = true
		end
	end
	
	if not found then
		table.insert(zomg_skills, {
			id=skill_id,
			totaldmg=dmg,
			count=1,
			highscore=dmg
		})
	end
end

function print_skill_stats()
	print(string.format(lang[language].known_skills, #zomg_skills))
	
	for index, skill in pairs(zomg_skills) do
			ability = Inspect.Ability.Detail(skill.id)
			
			if ability then
				avg = math.floor(skill.totaldmg / skill.count * 100)/100

				print(
					string.format(
						lang[language].stats, 	ability.name, skill.highscore,
						avg, skill.count
					)
				)
			end
	end
end

function reset()
	print(lang[language].reset)
	zomg_highscore = {}
	zomg_skills = {}
	zomg_count = 0
	zomg_totaldmg = 0
end

---------------------------
-- important functions ----
---------------------------

function init()
	table.insert(Event.Combat.Damage, {update, "zomg", "zomg.update.damage"})
	table.insert(Event.Combat.Heal, {update, "zomg", "zomg.update.heal"})
	table.insert(Event.Addon.SavedVariables.Load.End, {var_loaded, "zomg", "zomg.var_loaded"})
	table.insert(Command.Slash.Register("zomg"), {slash_register, "zomg", "zomg.slash_register"})
end

init()
print(lang[language].loaded_message)